package org.relaxng.datatype;

/**
 * Datatype object.
 * 
 * Every datatype object has several functionalities. Particularly,
 * 
 * <ol>
 *  <li> ability to identifies a class of character sequences. This is
 *       done through allows method.
 * 
 *  <li> ability to produce "value object" from a character sequence and
 *		 a context information.
 * 
 *  <li> ability to test equality of two value objects.
 * </ol>
 * 
 * This interface also defines optional diagnose method, which provides
 * human-friendly error diagnosis.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public interface DataType {
	/**
	 * checks if 'literal' matchs this datatype under the current context.
	 * 
	 * @param literal
	 *		the lexical representation to be verified
	 * @param context
	 *		context information that may be
	 *		necessary to verify the given literal.
	 * 
	 * @return
	 *		true if 'literal' is a member of this datatype;
	 *		false if it's not a member of this datatype.
	 */
	boolean accept( String literal, ContextProvider context );
	
	/**
	 * diagnoses the reason of error.
	 * 
	 * Application can call this method to provide detailed error message to users.
	 * This method is kept separate from verify method to achieve higher performance
	 * during normal validation. This method is optional and may not be implemented
	 * by some of datatypes.
	 * 
	 * @return null
	 *		if 'content' is accepted by this pattern.
	 * 
	 * @exception UnsupportedOperationException
	 *		if diagnosis is not supported by the implementation.
	 */
	DataTypeErrorDiagnosis diagnose( String literal, ContextProvider context )
		throws UnsupportedOperationException;
	
	/**
	 * gets type name suitable for display.
	 * 
	 * the callee cannot return null.
	 */
	String displayName();
	
	/**
	 * converts lexcial value and the current context to the corresponding
	 * value object.
	 * 
	 * @return	null
	 *		when the given lexical value is not a valid lexical value for this type.
	 */
	Object convertToValueObject( String lexicalValue, ContextProvider context );
	
	/**
	 * tests the equality of two value objects which belongs to this type.
	 * 
	 * The bahavior is undefined if objects not belonged to this type is passed.
	 * It is the caller's responsibility to ensure that objects
	 * belong to this type in fact.
	 * 
	 * @return
	 *		true if two value objects are considered equal;
	 *		false if otherwise.
	 */
	boolean testValueEquality( Object value1, Object value2 );
}
